# Copyright Amazon.com Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0
# Do not modify this file. This file is machine generated, and any changes to it will be overwritten.

import _dafny
from aws_cryptography_internal_kms.internaldafny.generated.ComAmazonawsKmsTypes import (
    CancelKeyDeletionRequest_CancelKeyDeletionRequest as DafnyCancelKeyDeletionRequest,
    CancelKeyDeletionResponse_CancelKeyDeletionResponse as DafnyCancelKeyDeletionResponse,
    ConnectCustomKeyStoreRequest_ConnectCustomKeyStoreRequest as DafnyConnectCustomKeyStoreRequest,
    ConnectCustomKeyStoreResponse_ConnectCustomKeyStoreResponse as DafnyConnectCustomKeyStoreResponse,
    CreateAliasRequest_CreateAliasRequest as DafnyCreateAliasRequest,
    CreateCustomKeyStoreRequest_CreateCustomKeyStoreRequest as DafnyCreateCustomKeyStoreRequest,
    CreateCustomKeyStoreResponse_CreateCustomKeyStoreResponse as DafnyCreateCustomKeyStoreResponse,
    CreateGrantRequest_CreateGrantRequest as DafnyCreateGrantRequest,
    CreateGrantResponse_CreateGrantResponse as DafnyCreateGrantResponse,
    CreateKeyRequest_CreateKeyRequest as DafnyCreateKeyRequest,
    CreateKeyResponse_CreateKeyResponse as DafnyCreateKeyResponse,
    DecryptRequest_DecryptRequest as DafnyDecryptRequest,
    DecryptResponse_DecryptResponse as DafnyDecryptResponse,
    DeleteAliasRequest_DeleteAliasRequest as DafnyDeleteAliasRequest,
    DeleteCustomKeyStoreRequest_DeleteCustomKeyStoreRequest as DafnyDeleteCustomKeyStoreRequest,
    DeleteCustomKeyStoreResponse_DeleteCustomKeyStoreResponse as DafnyDeleteCustomKeyStoreResponse,
    DeleteImportedKeyMaterialRequest_DeleteImportedKeyMaterialRequest as DafnyDeleteImportedKeyMaterialRequest,
    DeriveSharedSecretRequest_DeriveSharedSecretRequest as DafnyDeriveSharedSecretRequest,
    DeriveSharedSecretResponse_DeriveSharedSecretResponse as DafnyDeriveSharedSecretResponse,
    DescribeCustomKeyStoresRequest_DescribeCustomKeyStoresRequest as DafnyDescribeCustomKeyStoresRequest,
    DescribeCustomKeyStoresResponse_DescribeCustomKeyStoresResponse as DafnyDescribeCustomKeyStoresResponse,
    DescribeKeyRequest_DescribeKeyRequest as DafnyDescribeKeyRequest,
    DescribeKeyResponse_DescribeKeyResponse as DafnyDescribeKeyResponse,
    DisableKeyRequest_DisableKeyRequest as DafnyDisableKeyRequest,
    DisableKeyRotationRequest_DisableKeyRotationRequest as DafnyDisableKeyRotationRequest,
    DisconnectCustomKeyStoreRequest_DisconnectCustomKeyStoreRequest as DafnyDisconnectCustomKeyStoreRequest,
    DisconnectCustomKeyStoreResponse_DisconnectCustomKeyStoreResponse as DafnyDisconnectCustomKeyStoreResponse,
    EnableKeyRequest_EnableKeyRequest as DafnyEnableKeyRequest,
    EnableKeyRotationRequest_EnableKeyRotationRequest as DafnyEnableKeyRotationRequest,
    EncryptRequest_EncryptRequest as DafnyEncryptRequest,
    EncryptResponse_EncryptResponse as DafnyEncryptResponse,
    GenerateDataKeyPairRequest_GenerateDataKeyPairRequest as DafnyGenerateDataKeyPairRequest,
    GenerateDataKeyPairResponse_GenerateDataKeyPairResponse as DafnyGenerateDataKeyPairResponse,
    GenerateDataKeyPairWithoutPlaintextRequest_GenerateDataKeyPairWithoutPlaintextRequest as DafnyGenerateDataKeyPairWithoutPlaintextRequest,
    GenerateDataKeyPairWithoutPlaintextResponse_GenerateDataKeyPairWithoutPlaintextResponse as DafnyGenerateDataKeyPairWithoutPlaintextResponse,
    GenerateDataKeyRequest_GenerateDataKeyRequest as DafnyGenerateDataKeyRequest,
    GenerateDataKeyResponse_GenerateDataKeyResponse as DafnyGenerateDataKeyResponse,
    GenerateDataKeyWithoutPlaintextRequest_GenerateDataKeyWithoutPlaintextRequest as DafnyGenerateDataKeyWithoutPlaintextRequest,
    GenerateDataKeyWithoutPlaintextResponse_GenerateDataKeyWithoutPlaintextResponse as DafnyGenerateDataKeyWithoutPlaintextResponse,
    GenerateMacRequest_GenerateMacRequest as DafnyGenerateMacRequest,
    GenerateMacResponse_GenerateMacResponse as DafnyGenerateMacResponse,
    GenerateRandomRequest_GenerateRandomRequest as DafnyGenerateRandomRequest,
    GenerateRandomResponse_GenerateRandomResponse as DafnyGenerateRandomResponse,
    GetKeyPolicyRequest_GetKeyPolicyRequest as DafnyGetKeyPolicyRequest,
    GetKeyPolicyResponse_GetKeyPolicyResponse as DafnyGetKeyPolicyResponse,
    GetKeyRotationStatusRequest_GetKeyRotationStatusRequest as DafnyGetKeyRotationStatusRequest,
    GetKeyRotationStatusResponse_GetKeyRotationStatusResponse as DafnyGetKeyRotationStatusResponse,
    GetParametersForImportRequest_GetParametersForImportRequest as DafnyGetParametersForImportRequest,
    GetParametersForImportResponse_GetParametersForImportResponse as DafnyGetParametersForImportResponse,
    GetPublicKeyRequest_GetPublicKeyRequest as DafnyGetPublicKeyRequest,
    GetPublicKeyResponse_GetPublicKeyResponse as DafnyGetPublicKeyResponse,
    ImportKeyMaterialRequest_ImportKeyMaterialRequest as DafnyImportKeyMaterialRequest,
    ImportKeyMaterialResponse_ImportKeyMaterialResponse as DafnyImportKeyMaterialResponse,
    ListAliasesRequest_ListAliasesRequest as DafnyListAliasesRequest,
    ListAliasesResponse_ListAliasesResponse as DafnyListAliasesResponse,
    ListGrantsRequest_ListGrantsRequest as DafnyListGrantsRequest,
    ListGrantsResponse_ListGrantsResponse as DafnyListGrantsResponse,
    ListKeyPoliciesRequest_ListKeyPoliciesRequest as DafnyListKeyPoliciesRequest,
    ListKeyPoliciesResponse_ListKeyPoliciesResponse as DafnyListKeyPoliciesResponse,
    ListKeyRotationsRequest_ListKeyRotationsRequest as DafnyListKeyRotationsRequest,
    ListKeyRotationsResponse_ListKeyRotationsResponse as DafnyListKeyRotationsResponse,
    ListKeysRequest_ListKeysRequest as DafnyListKeysRequest,
    ListKeysResponse_ListKeysResponse as DafnyListKeysResponse,
    ListResourceTagsRequest_ListResourceTagsRequest as DafnyListResourceTagsRequest,
    ListResourceTagsResponse_ListResourceTagsResponse as DafnyListResourceTagsResponse,
    PutKeyPolicyRequest_PutKeyPolicyRequest as DafnyPutKeyPolicyRequest,
    ReEncryptRequest_ReEncryptRequest as DafnyReEncryptRequest,
    ReEncryptResponse_ReEncryptResponse as DafnyReEncryptResponse,
    ReplicateKeyRequest_ReplicateKeyRequest as DafnyReplicateKeyRequest,
    ReplicateKeyResponse_ReplicateKeyResponse as DafnyReplicateKeyResponse,
    RetireGrantRequest_RetireGrantRequest as DafnyRetireGrantRequest,
    RevokeGrantRequest_RevokeGrantRequest as DafnyRevokeGrantRequest,
    RotateKeyOnDemandRequest_RotateKeyOnDemandRequest as DafnyRotateKeyOnDemandRequest,
    RotateKeyOnDemandResponse_RotateKeyOnDemandResponse as DafnyRotateKeyOnDemandResponse,
    ScheduleKeyDeletionRequest_ScheduleKeyDeletionRequest as DafnyScheduleKeyDeletionRequest,
    ScheduleKeyDeletionResponse_ScheduleKeyDeletionResponse as DafnyScheduleKeyDeletionResponse,
    SignRequest_SignRequest as DafnySignRequest,
    SignResponse_SignResponse as DafnySignResponse,
    TagResourceRequest_TagResourceRequest as DafnyTagResourceRequest,
    UntagResourceRequest_UntagResourceRequest as DafnyUntagResourceRequest,
    UpdateAliasRequest_UpdateAliasRequest as DafnyUpdateAliasRequest,
    UpdateCustomKeyStoreRequest_UpdateCustomKeyStoreRequest as DafnyUpdateCustomKeyStoreRequest,
    UpdateCustomKeyStoreResponse_UpdateCustomKeyStoreResponse as DafnyUpdateCustomKeyStoreResponse,
    UpdateKeyDescriptionRequest_UpdateKeyDescriptionRequest as DafnyUpdateKeyDescriptionRequest,
    UpdatePrimaryRegionRequest_UpdatePrimaryRegionRequest as DafnyUpdatePrimaryRegionRequest,
    VerifyMacRequest_VerifyMacRequest as DafnyVerifyMacRequest,
    VerifyMacResponse_VerifyMacResponse as DafnyVerifyMacResponse,
    VerifyRequest_VerifyRequest as DafnyVerifyRequest,
    VerifyResponse_VerifyResponse as DafnyVerifyResponse,
)
import aws_cryptography_internal_kms.internaldafny.generated.module_
import aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny
import aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk

from . import dafny_to_aws_sdk


import smithy_dafny_standard_library.internaldafny.generated.Wrappers as Wrappers
from botocore.exceptions import ClientError
import aws_cryptography_internal_kms.internaldafny.generated.ComAmazonawsKmsTypes


def _sdk_error_to_dafny_error(e: ClientError):
    """Converts the provided native Smithy-modelled error into the
    corresponding Dafny error."""
    if e.response["Error"]["Code"] == "AlreadyExistsException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_AlreadyExistsException(
            e.response
        )

    elif e.response["Error"]["Code"] == "CloudHsmClusterInUseException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_CloudHsmClusterInUseException(
            e.response
        )

    elif e.response["Error"]["Code"] == "CloudHsmClusterInvalidConfigurationException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_CloudHsmClusterInvalidConfigurationException(
            e.response
        )

    elif e.response["Error"]["Code"] == "CloudHsmClusterNotActiveException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_CloudHsmClusterNotActiveException(
            e.response
        )

    elif e.response["Error"]["Code"] == "CloudHsmClusterNotFoundException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_CloudHsmClusterNotFoundException(
            e.response
        )

    elif e.response["Error"]["Code"] == "CloudHsmClusterNotRelatedException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_CloudHsmClusterNotRelatedException(
            e.response
        )

    elif e.response["Error"]["Code"] == "ConflictException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_ConflictException(
            e.response
        )

    elif e.response["Error"]["Code"] == "CustomKeyStoreHasCMKsException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_CustomKeyStoreHasCMKsException(
            e.response
        )

    elif e.response["Error"]["Code"] == "CustomKeyStoreInvalidStateException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_CustomKeyStoreInvalidStateException(
            e.response
        )

    elif e.response["Error"]["Code"] == "CustomKeyStoreNameInUseException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_CustomKeyStoreNameInUseException(
            e.response
        )

    elif e.response["Error"]["Code"] == "CustomKeyStoreNotFoundException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_CustomKeyStoreNotFoundException(
            e.response
        )

    elif e.response["Error"]["Code"] == "DependencyTimeoutException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_DependencyTimeoutException(
            e.response
        )

    elif e.response["Error"]["Code"] == "DisabledException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_DisabledException(
            e.response
        )

    elif e.response["Error"]["Code"] == "DryRunOperationException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_DryRunOperationException(
            e.response
        )

    elif e.response["Error"]["Code"] == "ExpiredImportTokenException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_ExpiredImportTokenException(
            e.response
        )

    elif e.response["Error"]["Code"] == "IncorrectKeyException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_IncorrectKeyException(
            e.response
        )

    elif e.response["Error"]["Code"] == "IncorrectKeyMaterialException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_IncorrectKeyMaterialException(
            e.response
        )

    elif e.response["Error"]["Code"] == "IncorrectTrustAnchorException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_IncorrectTrustAnchorException(
            e.response
        )

    elif e.response["Error"]["Code"] == "InvalidAliasNameException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_InvalidAliasNameException(
            e.response
        )

    elif e.response["Error"]["Code"] == "InvalidArnException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_InvalidArnException(
            e.response
        )

    elif e.response["Error"]["Code"] == "InvalidCiphertextException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_InvalidCiphertextException(
            e.response
        )

    elif e.response["Error"]["Code"] == "InvalidGrantIdException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_InvalidGrantIdException(
            e.response
        )

    elif e.response["Error"]["Code"] == "InvalidGrantTokenException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_InvalidGrantTokenException(
            e.response
        )

    elif e.response["Error"]["Code"] == "InvalidImportTokenException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_InvalidImportTokenException(
            e.response
        )

    elif e.response["Error"]["Code"] == "InvalidKeyUsageException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_InvalidKeyUsageException(
            e.response
        )

    elif e.response["Error"]["Code"] == "InvalidMarkerException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_InvalidMarkerException(
            e.response
        )

    elif e.response["Error"]["Code"] == "KeyUnavailableException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_KeyUnavailableException(
            e.response
        )

    elif e.response["Error"]["Code"] == "KMSInternalException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_KMSInternalException(
            e.response
        )

    elif e.response["Error"]["Code"] == "KMSInvalidMacException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_KMSInvalidMacException(
            e.response
        )

    elif e.response["Error"]["Code"] == "KMSInvalidSignatureException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_KMSInvalidSignatureException(
            e.response
        )

    elif e.response["Error"]["Code"] == "KMSInvalidStateException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_KMSInvalidStateException(
            e.response
        )

    elif e.response["Error"]["Code"] == "LimitExceededException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_LimitExceededException(
            e.response
        )

    elif e.response["Error"]["Code"] == "MalformedPolicyDocumentException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_MalformedPolicyDocumentException(
            e.response
        )

    elif e.response["Error"]["Code"] == "NotFoundException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_NotFoundException(
            e.response
        )

    elif e.response["Error"]["Code"] == "TagException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_TagException(
            e.response
        )

    elif e.response["Error"]["Code"] == "UnsupportedOperationException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_UnsupportedOperationException(
            e.response
        )

    elif e.response["Error"]["Code"] == "XksKeyAlreadyInUseException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_XksKeyAlreadyInUseException(
            e.response
        )

    elif e.response["Error"]["Code"] == "XksKeyInvalidConfigurationException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_XksKeyInvalidConfigurationException(
            e.response
        )

    elif e.response["Error"]["Code"] == "XksKeyNotFoundException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_XksKeyNotFoundException(
            e.response
        )

    elif (
        e.response["Error"]["Code"]
        == "XksProxyIncorrectAuthenticationCredentialException"
    ):
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_XksProxyIncorrectAuthenticationCredentialException(
            e.response
        )

    elif e.response["Error"]["Code"] == "XksProxyInvalidConfigurationException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_XksProxyInvalidConfigurationException(
            e.response
        )

    elif e.response["Error"]["Code"] == "XksProxyInvalidResponseException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_XksProxyInvalidResponseException(
            e.response
        )

    elif e.response["Error"]["Code"] == "XksProxyUriEndpointInUseException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_XksProxyUriEndpointInUseException(
            e.response
        )

    elif e.response["Error"]["Code"] == "XksProxyUriInUseException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_XksProxyUriInUseException(
            e.response
        )

    elif e.response["Error"]["Code"] == "XksProxyUriUnreachableException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_XksProxyUriUnreachableException(
            e.response
        )

    elif e.response["Error"]["Code"] == "XksProxyVpcEndpointServiceInUseException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_XksProxyVpcEndpointServiceInUseException(
            e.response
        )

    elif (
        e.response["Error"]["Code"]
        == "XksProxyVpcEndpointServiceInvalidConfigurationException"
    ):
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_XksProxyVpcEndpointServiceInvalidConfigurationException(
            e.response
        )

    elif e.response["Error"]["Code"] == "XksProxyVpcEndpointServiceNotFoundException":
        return aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_XksProxyVpcEndpointServiceNotFoundException(
            e.response
        )

    return aws_cryptography_internal_kms.internaldafny.generated.ComAmazonawsKmsTypes.Error_Opaque(
        obj=e,
        alt__text=_dafny.Seq(
            "".join(
                [
                    chr(int.from_bytes(pair, "big"))
                    for pair in zip(*[iter(repr(e).encode("utf-16-be"))] * 2)
                ]
            )
        ),
    )


class KMSClientShim:
    def __init__(self, _impl, _region):
        self._impl = _impl
        self._region = _region

    def CancelKeyDeletion(
        self, input: DafnyCancelKeyDeletionRequest
    ) -> DafnyCancelKeyDeletionResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_CancelKeyDeletionRequest(
            input
        )
        try:
            boto_response_dict = self._impl.cancel_key_deletion(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_CancelKeyDeletionResponse(
                boto_response_dict
            )
        )

    def ConnectCustomKeyStore(
        self, input: DafnyConnectCustomKeyStoreRequest
    ) -> DafnyConnectCustomKeyStoreResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_ConnectCustomKeyStoreRequest(
            input
        )
        try:
            boto_response_dict = self._impl.connect_custom_key_store(
                **boto_request_dict
            )
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_ConnectCustomKeyStoreResponse(
                boto_response_dict
            )
        )

    def CreateAlias(self, input: DafnyCreateAliasRequest) -> None:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_CreateAliasRequest(
            input
        )
        try:
            boto_response_dict = self._impl.create_alias(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(None)

    def CreateCustomKeyStore(
        self, input: DafnyCreateCustomKeyStoreRequest
    ) -> DafnyCreateCustomKeyStoreResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_CreateCustomKeyStoreRequest(
            input
        )
        try:
            boto_response_dict = self._impl.create_custom_key_store(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_CreateCustomKeyStoreResponse(
                boto_response_dict
            )
        )

    def CreateGrant(self, input: DafnyCreateGrantRequest) -> DafnyCreateGrantResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_CreateGrantRequest(
            input
        )
        try:
            boto_response_dict = self._impl.create_grant(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_CreateGrantResponse(
                boto_response_dict
            )
        )

    def CreateKey(self, input: DafnyCreateKeyRequest) -> DafnyCreateKeyResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_CreateKeyRequest(
            input
        )
        try:
            boto_response_dict = self._impl.create_key(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_CreateKeyResponse(
                boto_response_dict
            )
        )

    def Decrypt(self, input: DafnyDecryptRequest) -> DafnyDecryptResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_DecryptRequest(
            input
        )
        try:
            boto_response_dict = self._impl.decrypt(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_DecryptResponse(
                boto_response_dict
            )
        )

    def DeleteAlias(self, input: DafnyDeleteAliasRequest) -> None:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_DeleteAliasRequest(
            input
        )
        try:
            boto_response_dict = self._impl.delete_alias(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(None)

    def DeleteCustomKeyStore(
        self, input: DafnyDeleteCustomKeyStoreRequest
    ) -> DafnyDeleteCustomKeyStoreResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_DeleteCustomKeyStoreRequest(
            input
        )
        try:
            boto_response_dict = self._impl.delete_custom_key_store(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_DeleteCustomKeyStoreResponse(
                boto_response_dict
            )
        )

    def DeleteImportedKeyMaterial(
        self, input: DafnyDeleteImportedKeyMaterialRequest
    ) -> None:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_DeleteImportedKeyMaterialRequest(
            input
        )
        try:
            boto_response_dict = self._impl.delete_imported_key_material(
                **boto_request_dict
            )
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(None)

    def DeriveSharedSecret(
        self, input: DafnyDeriveSharedSecretRequest
    ) -> DafnyDeriveSharedSecretResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_DeriveSharedSecretRequest(
            input
        )
        try:
            boto_response_dict = self._impl.derive_shared_secret(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_DeriveSharedSecretResponse(
                boto_response_dict
            )
        )

    def DescribeCustomKeyStores(
        self, input: DafnyDescribeCustomKeyStoresRequest
    ) -> DafnyDescribeCustomKeyStoresResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_DescribeCustomKeyStoresRequest(
            input
        )
        try:
            boto_response_dict = self._impl.describe_custom_key_stores(
                **boto_request_dict
            )
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_DescribeCustomKeyStoresResponse(
                boto_response_dict
            )
        )

    def DescribeKey(self, input: DafnyDescribeKeyRequest) -> DafnyDescribeKeyResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_DescribeKeyRequest(
            input
        )
        try:
            boto_response_dict = self._impl.describe_key(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_DescribeKeyResponse(
                boto_response_dict
            )
        )

    def DisableKey(self, input: DafnyDisableKeyRequest) -> None:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_DisableKeyRequest(
            input
        )
        try:
            boto_response_dict = self._impl.disable_key(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(None)

    def DisableKeyRotation(self, input: DafnyDisableKeyRotationRequest) -> None:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_DisableKeyRotationRequest(
            input
        )
        try:
            boto_response_dict = self._impl.disable_key_rotation(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(None)

    def DisconnectCustomKeyStore(
        self, input: DafnyDisconnectCustomKeyStoreRequest
    ) -> DafnyDisconnectCustomKeyStoreResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_DisconnectCustomKeyStoreRequest(
            input
        )
        try:
            boto_response_dict = self._impl.disconnect_custom_key_store(
                **boto_request_dict
            )
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_DisconnectCustomKeyStoreResponse(
                boto_response_dict
            )
        )

    def EnableKey(self, input: DafnyEnableKeyRequest) -> None:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_EnableKeyRequest(
            input
        )
        try:
            boto_response_dict = self._impl.enable_key(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(None)

    def EnableKeyRotation(self, input: DafnyEnableKeyRotationRequest) -> None:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_EnableKeyRotationRequest(
            input
        )
        try:
            boto_response_dict = self._impl.enable_key_rotation(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(None)

    def Encrypt(self, input: DafnyEncryptRequest) -> DafnyEncryptResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_EncryptRequest(
            input
        )
        try:
            boto_response_dict = self._impl.encrypt(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_EncryptResponse(
                boto_response_dict
            )
        )

    def GenerateDataKey(
        self, input: DafnyGenerateDataKeyRequest
    ) -> DafnyGenerateDataKeyResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_GenerateDataKeyRequest(
            input
        )
        try:
            boto_response_dict = self._impl.generate_data_key(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_GenerateDataKeyResponse(
                boto_response_dict
            )
        )

    def GenerateDataKeyPair(
        self, input: DafnyGenerateDataKeyPairRequest
    ) -> DafnyGenerateDataKeyPairResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_GenerateDataKeyPairRequest(
            input
        )
        try:
            boto_response_dict = self._impl.generate_data_key_pair(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_GenerateDataKeyPairResponse(
                boto_response_dict
            )
        )

    def GenerateDataKeyPairWithoutPlaintext(
        self, input: DafnyGenerateDataKeyPairWithoutPlaintextRequest
    ) -> DafnyGenerateDataKeyPairWithoutPlaintextResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_GenerateDataKeyPairWithoutPlaintextRequest(
            input
        )
        try:
            boto_response_dict = self._impl.generate_data_key_pair_without_plaintext(
                **boto_request_dict
            )
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_GenerateDataKeyPairWithoutPlaintextResponse(
                boto_response_dict
            )
        )

    def GenerateDataKeyWithoutPlaintext(
        self, input: DafnyGenerateDataKeyWithoutPlaintextRequest
    ) -> DafnyGenerateDataKeyWithoutPlaintextResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_GenerateDataKeyWithoutPlaintextRequest(
            input
        )
        try:
            boto_response_dict = self._impl.generate_data_key_without_plaintext(
                **boto_request_dict
            )
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_GenerateDataKeyWithoutPlaintextResponse(
                boto_response_dict
            )
        )

    def GenerateMac(self, input: DafnyGenerateMacRequest) -> DafnyGenerateMacResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_GenerateMacRequest(
            input
        )
        try:
            boto_response_dict = self._impl.generate_mac(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_GenerateMacResponse(
                boto_response_dict
            )
        )

    def GenerateRandom(
        self, input: DafnyGenerateRandomRequest
    ) -> DafnyGenerateRandomResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_GenerateRandomRequest(
            input
        )
        try:
            boto_response_dict = self._impl.generate_random(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_GenerateRandomResponse(
                boto_response_dict
            )
        )

    def GetKeyPolicy(
        self, input: DafnyGetKeyPolicyRequest
    ) -> DafnyGetKeyPolicyResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_GetKeyPolicyRequest(
            input
        )
        try:
            boto_response_dict = self._impl.get_key_policy(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_GetKeyPolicyResponse(
                boto_response_dict
            )
        )

    def GetKeyRotationStatus(
        self, input: DafnyGetKeyRotationStatusRequest
    ) -> DafnyGetKeyRotationStatusResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_GetKeyRotationStatusRequest(
            input
        )
        try:
            boto_response_dict = self._impl.get_key_rotation_status(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_GetKeyRotationStatusResponse(
                boto_response_dict
            )
        )

    def GetParametersForImport(
        self, input: DafnyGetParametersForImportRequest
    ) -> DafnyGetParametersForImportResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_GetParametersForImportRequest(
            input
        )
        try:
            boto_response_dict = self._impl.get_parameters_for_import(
                **boto_request_dict
            )
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_GetParametersForImportResponse(
                boto_response_dict
            )
        )

    def GetPublicKey(
        self, input: DafnyGetPublicKeyRequest
    ) -> DafnyGetPublicKeyResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_GetPublicKeyRequest(
            input
        )
        try:
            boto_response_dict = self._impl.get_public_key(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_GetPublicKeyResponse(
                boto_response_dict
            )
        )

    def ImportKeyMaterial(
        self, input: DafnyImportKeyMaterialRequest
    ) -> DafnyImportKeyMaterialResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_ImportKeyMaterialRequest(
            input
        )
        try:
            boto_response_dict = self._impl.import_key_material(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_ImportKeyMaterialResponse(
                boto_response_dict
            )
        )

    def ListAliases(self, input: DafnyListAliasesRequest) -> DafnyListAliasesResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_ListAliasesRequest(
            input
        )
        try:
            boto_response_dict = self._impl.list_aliases(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_ListAliasesResponse(
                boto_response_dict
            )
        )

    def ListGrants(self, input: DafnyListGrantsRequest) -> DafnyListGrantsResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_ListGrantsRequest(
            input
        )
        try:
            boto_response_dict = self._impl.list_grants(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_ListGrantsResponse(
                boto_response_dict
            )
        )

    def ListKeyPolicies(
        self, input: DafnyListKeyPoliciesRequest
    ) -> DafnyListKeyPoliciesResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_ListKeyPoliciesRequest(
            input
        )
        try:
            boto_response_dict = self._impl.list_key_policies(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_ListKeyPoliciesResponse(
                boto_response_dict
            )
        )

    def ListKeyRotations(
        self, input: DafnyListKeyRotationsRequest
    ) -> DafnyListKeyRotationsResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_ListKeyRotationsRequest(
            input
        )
        try:
            boto_response_dict = self._impl.list_key_rotations(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_ListKeyRotationsResponse(
                boto_response_dict
            )
        )

    def ListKeys(self, input: DafnyListKeysRequest) -> DafnyListKeysResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_ListKeysRequest(
            input
        )
        try:
            boto_response_dict = self._impl.list_keys(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_ListKeysResponse(
                boto_response_dict
            )
        )

    def ListResourceTags(
        self, input: DafnyListResourceTagsRequest
    ) -> DafnyListResourceTagsResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_ListResourceTagsRequest(
            input
        )
        try:
            boto_response_dict = self._impl.list_resource_tags(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_ListResourceTagsResponse(
                boto_response_dict
            )
        )

    def PutKeyPolicy(self, input: DafnyPutKeyPolicyRequest) -> None:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_PutKeyPolicyRequest(
            input
        )
        try:
            boto_response_dict = self._impl.put_key_policy(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(None)

    def ReEncrypt(self, input: DafnyReEncryptRequest) -> DafnyReEncryptResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_ReEncryptRequest(
            input
        )
        try:
            boto_response_dict = self._impl.re_encrypt(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_ReEncryptResponse(
                boto_response_dict
            )
        )

    def ReplicateKey(
        self, input: DafnyReplicateKeyRequest
    ) -> DafnyReplicateKeyResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_ReplicateKeyRequest(
            input
        )
        try:
            boto_response_dict = self._impl.replicate_key(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_ReplicateKeyResponse(
                boto_response_dict
            )
        )

    def RetireGrant(self, input: DafnyRetireGrantRequest) -> None:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_RetireGrantRequest(
            input
        )
        try:
            boto_response_dict = self._impl.retire_grant(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(None)

    def RevokeGrant(self, input: DafnyRevokeGrantRequest) -> None:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_RevokeGrantRequest(
            input
        )
        try:
            boto_response_dict = self._impl.revoke_grant(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(None)

    def RotateKeyOnDemand(
        self, input: DafnyRotateKeyOnDemandRequest
    ) -> DafnyRotateKeyOnDemandResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_RotateKeyOnDemandRequest(
            input
        )
        try:
            boto_response_dict = self._impl.rotate_key_on_demand(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_RotateKeyOnDemandResponse(
                boto_response_dict
            )
        )

    def ScheduleKeyDeletion(
        self, input: DafnyScheduleKeyDeletionRequest
    ) -> DafnyScheduleKeyDeletionResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_ScheduleKeyDeletionRequest(
            input
        )
        try:
            boto_response_dict = self._impl.schedule_key_deletion(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_ScheduleKeyDeletionResponse(
                boto_response_dict
            )
        )

    def Sign(self, input: DafnySignRequest) -> DafnySignResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_SignRequest(
            input
        )
        try:
            boto_response_dict = self._impl.sign(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_SignResponse(
                boto_response_dict
            )
        )

    def TagResource(self, input: DafnyTagResourceRequest) -> None:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_TagResourceRequest(
            input
        )
        try:
            boto_response_dict = self._impl.tag_resource(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(None)

    def UntagResource(self, input: DafnyUntagResourceRequest) -> None:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_UntagResourceRequest(
            input
        )
        try:
            boto_response_dict = self._impl.untag_resource(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(None)

    def UpdateAlias(self, input: DafnyUpdateAliasRequest) -> None:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_UpdateAliasRequest(
            input
        )
        try:
            boto_response_dict = self._impl.update_alias(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(None)

    def UpdateCustomKeyStore(
        self, input: DafnyUpdateCustomKeyStoreRequest
    ) -> DafnyUpdateCustomKeyStoreResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_UpdateCustomKeyStoreRequest(
            input
        )
        try:
            boto_response_dict = self._impl.update_custom_key_store(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_UpdateCustomKeyStoreResponse(
                boto_response_dict
            )
        )

    def UpdateKeyDescription(self, input: DafnyUpdateKeyDescriptionRequest) -> None:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_UpdateKeyDescriptionRequest(
            input
        )
        try:
            boto_response_dict = self._impl.update_key_description(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(None)

    def UpdatePrimaryRegion(self, input: DafnyUpdatePrimaryRegionRequest) -> None:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_UpdatePrimaryRegionRequest(
            input
        )
        try:
            boto_response_dict = self._impl.update_primary_region(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(None)

    def Verify(self, input: DafnyVerifyRequest) -> DafnyVerifyResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_VerifyRequest(
            input
        )
        try:
            boto_response_dict = self._impl.verify(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_VerifyResponse(
                boto_response_dict
            )
        )

    def VerifyMac(self, input: DafnyVerifyMacRequest) -> DafnyVerifyMacResponse:
        boto_request_dict = aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.dafny_to_aws_sdk.com_amazonaws_kms_VerifyMacRequest(
            input
        )
        try:
            boto_response_dict = self._impl.verify_mac(**boto_request_dict)
        except ClientError as e:
            return Wrappers.Result_Failure(_sdk_error_to_dafny_error(e))

        return Wrappers.Result_Success(
            aws_cryptography_internal_kms.smithygenerated.com_amazonaws_kms.aws_sdk_to_dafny.com_amazonaws_kms_VerifyMacResponse(
                boto_response_dict
            )
        )



# Copyright Amazon.com Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0

# Initialize generated Dafny
from .internaldafny.generated import module_

# Initialize externs
from .internaldafny import extern



import sys
from typing import Callable, Any, TypeVar, NamedTuple
from math import floor
from itertools import count

import smithy_dafny_standard_library.internaldafny.generated.module_ as module_
import _dafny as _dafny
import System_ as System_
import smithy_dafny_standard_library.internaldafny.generated.Wrappers as Wrappers
import smithy_dafny_standard_library.internaldafny.generated.Relations as Relations
import smithy_dafny_standard_library.internaldafny.generated.Seq_MergeSort as Seq_MergeSort
import smithy_dafny_standard_library.internaldafny.generated.Math as Math
import smithy_dafny_standard_library.internaldafny.generated.Seq as Seq
import smithy_dafny_standard_library.internaldafny.generated.BoundedInts as BoundedInts
import smithy_dafny_standard_library.internaldafny.generated.Unicode as Unicode
import smithy_dafny_standard_library.internaldafny.generated.Functions as Functions
import smithy_dafny_standard_library.internaldafny.generated.Utf8EncodingForm as Utf8EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.Utf16EncodingForm as Utf16EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.UnicodeStrings as UnicodeStrings
import smithy_dafny_standard_library.internaldafny.generated.FileIO as FileIO
import smithy_dafny_standard_library.internaldafny.generated.GeneralInternals as GeneralInternals
import smithy_dafny_standard_library.internaldafny.generated.MulInternalsNonlinear as MulInternalsNonlinear

# Module: MulInternals

class default__:
    def  __init__(self):
        pass

    @staticmethod
    def MulPos(x, y):
        d_0___accumulator_ = 0
        while True:
            with _dafny.label():
                if (x) == (0):
                    return (0) + (d_0___accumulator_)
                elif True:
                    d_0___accumulator_ = (d_0___accumulator_) + (y)
                    in0_ = (x) - (1)
                    in1_ = y
                    x = in0_
                    y = in1_
                    raise _dafny.TailCall()
                break

    @staticmethod
    def MulRecursive(x, y):
        if (x) >= (0):
            return default__.MulPos(x, y)
        elif True:
            return (-1) * (default__.MulPos((-1) * (x), y))




import sys
from typing import Callable, Any, TypeVar, NamedTuple
from math import floor
from itertools import count

import smithy_dafny_standard_library.internaldafny.generated.module_ as module_
import _dafny as _dafny
import System_ as System_
import smithy_dafny_standard_library.internaldafny.generated.Wrappers as Wrappers
import smithy_dafny_standard_library.internaldafny.generated.Relations as Relations
import smithy_dafny_standard_library.internaldafny.generated.Seq_MergeSort as Seq_MergeSort
import smithy_dafny_standard_library.internaldafny.generated.Math as Math
import smithy_dafny_standard_library.internaldafny.generated.Seq as Seq
import smithy_dafny_standard_library.internaldafny.generated.BoundedInts as BoundedInts
import smithy_dafny_standard_library.internaldafny.generated.Unicode as Unicode
import smithy_dafny_standard_library.internaldafny.generated.Functions as Functions
import smithy_dafny_standard_library.internaldafny.generated.Utf8EncodingForm as Utf8EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.Utf16EncodingForm as Utf16EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.UnicodeStrings as UnicodeStrings
import smithy_dafny_standard_library.internaldafny.generated.FileIO as FileIO
import smithy_dafny_standard_library.internaldafny.generated.GeneralInternals as GeneralInternals
import smithy_dafny_standard_library.internaldafny.generated.MulInternalsNonlinear as MulInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.MulInternals as MulInternals
import smithy_dafny_standard_library.internaldafny.generated.Mul as Mul
import smithy_dafny_standard_library.internaldafny.generated.ModInternalsNonlinear as ModInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.DivInternalsNonlinear as DivInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.ModInternals as ModInternals
import smithy_dafny_standard_library.internaldafny.generated.DivInternals as DivInternals
import smithy_dafny_standard_library.internaldafny.generated.DivMod as DivMod
import smithy_dafny_standard_library.internaldafny.generated.Power as Power
import smithy_dafny_standard_library.internaldafny.generated.Logarithm as Logarithm
import smithy_dafny_standard_library.internaldafny.generated.StandardLibraryInterop as StandardLibraryInterop
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_UInt as StandardLibrary_UInt
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_String as StandardLibrary_String
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary as StandardLibrary
import smithy_dafny_standard_library.internaldafny.generated.UUID as UUID
import smithy_dafny_standard_library.internaldafny.generated.UTF8 as UTF8
import smithy_dafny_standard_library.internaldafny.generated.Time as Time
import smithy_dafny_standard_library.internaldafny.generated.Streams as Streams
import smithy_dafny_standard_library.internaldafny.generated.Sorting as Sorting
import smithy_dafny_standard_library.internaldafny.generated.SortedSets as SortedSets
import smithy_dafny_standard_library.internaldafny.generated.HexStrings as HexStrings
import smithy_dafny_standard_library.internaldafny.generated.GetOpt as GetOpt
import smithy_dafny_standard_library.internaldafny.generated.FloatCompare as FloatCompare
import smithy_dafny_standard_library.internaldafny.generated.ConcurrentCall as ConcurrentCall
import smithy_dafny_standard_library.internaldafny.generated.Base64 as Base64
import smithy_dafny_standard_library.internaldafny.generated.Base64Lemmas as Base64Lemmas
import smithy_dafny_standard_library.internaldafny.generated.Actions as Actions
import smithy_dafny_standard_library.internaldafny.generated.DafnyLibraries as DafnyLibraries
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views_Core as JSON_Utils_Views_Core
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views_Writers as JSON_Utils_Views_Writers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views as JSON_Utils_Views
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers_Core as JSON_Utils_Lexers_Core

# Module: JSON_Utils_Lexers_Strings

class default__:
    def  __init__(self):
        pass

    @staticmethod
    def StringBody(escaped, byte):
        if (byte) == (ord('\\')):
            return JSON_Utils_Lexers_Core.LexerResult_Partial(not(escaped))
        elif ((byte) == (ord('\"'))) and (not(escaped)):
            return JSON_Utils_Lexers_Core.LexerResult_Accept()
        elif True:
            return JSON_Utils_Lexers_Core.LexerResult_Partial(False)

    @staticmethod
    def String(st, byte):
        source0_ = st
        if True:
            if source0_.is_Start:
                if (byte) == (ord('\"')):
                    return JSON_Utils_Lexers_Core.LexerResult_Partial(StringLexerState_Body(False))
                elif True:
                    return JSON_Utils_Lexers_Core.LexerResult_Reject(_dafny.Seq("String must start with double quote"))
        if True:
            if source0_.is_End:
                return JSON_Utils_Lexers_Core.LexerResult_Accept()
        if True:
            d_0_escaped_ = source0_.escaped
            if (byte) == (ord('\\')):
                return JSON_Utils_Lexers_Core.LexerResult_Partial(StringLexerState_Body(not(d_0_escaped_)))
            elif ((byte) == (ord('\"'))) and (not(d_0_escaped_)):
                return JSON_Utils_Lexers_Core.LexerResult_Partial(StringLexerState_End())
            elif True:
                return JSON_Utils_Lexers_Core.LexerResult_Partial(StringLexerState_Body(False))

    @_dafny.classproperty
    def StringBodyLexerStart(instance):
        return False
    @_dafny.classproperty
    def StringLexerStart(instance):
        return StringLexerState_Start()

class StringLexerState:
    @classmethod
    def default(cls, ):
        return lambda: StringLexerState_Start()
    def __ne__(self, __o: object) -> bool:
        return not self.__eq__(__o)
    @property
    def is_Start(self) -> bool:
        return isinstance(self, StringLexerState_Start)
    @property
    def is_Body(self) -> bool:
        return isinstance(self, StringLexerState_Body)
    @property
    def is_End(self) -> bool:
        return isinstance(self, StringLexerState_End)

class StringLexerState_Start(StringLexerState, NamedTuple('Start', [])):
    def __dafnystr__(self) -> str:
        return f'Strings.StringLexerState.Start'
    def __eq__(self, __o: object) -> bool:
        return isinstance(__o, StringLexerState_Start)
    def __hash__(self) -> int:
        return super().__hash__()

class StringLexerState_Body(StringLexerState, NamedTuple('Body', [('escaped', Any)])):
    def __dafnystr__(self) -> str:
        return f'Strings.StringLexerState.Body({_dafny.string_of(self.escaped)})'
    def __eq__(self, __o: object) -> bool:
        return isinstance(__o, StringLexerState_Body) and self.escaped == __o.escaped
    def __hash__(self) -> int:
        return super().__hash__()

class StringLexerState_End(StringLexerState, NamedTuple('End', [])):
    def __dafnystr__(self) -> str:
        return f'Strings.StringLexerState.End'
    def __eq__(self, __o: object) -> bool:
        return isinstance(__o, StringLexerState_End)
    def __hash__(self) -> int:
        return super().__hash__()




import sys
from typing import Callable, Any, TypeVar, NamedTuple
from math import floor
from itertools import count

import smithy_dafny_standard_library.internaldafny.generated.module_ as module_
import _dafny as _dafny
import System_ as System_
import smithy_dafny_standard_library.internaldafny.generated.Wrappers as Wrappers
import smithy_dafny_standard_library.internaldafny.generated.Relations as Relations
import smithy_dafny_standard_library.internaldafny.generated.Seq_MergeSort as Seq_MergeSort
import smithy_dafny_standard_library.internaldafny.generated.Math as Math
import smithy_dafny_standard_library.internaldafny.generated.Seq as Seq
import smithy_dafny_standard_library.internaldafny.generated.BoundedInts as BoundedInts
import smithy_dafny_standard_library.internaldafny.generated.Unicode as Unicode
import smithy_dafny_standard_library.internaldafny.generated.Functions as Functions
import smithy_dafny_standard_library.internaldafny.generated.Utf8EncodingForm as Utf8EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.Utf16EncodingForm as Utf16EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.UnicodeStrings as UnicodeStrings
import smithy_dafny_standard_library.internaldafny.generated.FileIO as FileIO
import smithy_dafny_standard_library.internaldafny.generated.GeneralInternals as GeneralInternals
import smithy_dafny_standard_library.internaldafny.generated.MulInternalsNonlinear as MulInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.MulInternals as MulInternals
import smithy_dafny_standard_library.internaldafny.generated.Mul as Mul
import smithy_dafny_standard_library.internaldafny.generated.ModInternalsNonlinear as ModInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.DivInternalsNonlinear as DivInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.ModInternals as ModInternals
import smithy_dafny_standard_library.internaldafny.generated.DivInternals as DivInternals
import smithy_dafny_standard_library.internaldafny.generated.DivMod as DivMod
import smithy_dafny_standard_library.internaldafny.generated.Power as Power
import smithy_dafny_standard_library.internaldafny.generated.Logarithm as Logarithm
import smithy_dafny_standard_library.internaldafny.generated.StandardLibraryInterop as StandardLibraryInterop
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_UInt as StandardLibrary_UInt
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_String as StandardLibrary_String
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary as StandardLibrary
import smithy_dafny_standard_library.internaldafny.generated.UUID as UUID
import smithy_dafny_standard_library.internaldafny.generated.UTF8 as UTF8
import smithy_dafny_standard_library.internaldafny.generated.Time as Time
import smithy_dafny_standard_library.internaldafny.generated.Streams as Streams
import smithy_dafny_standard_library.internaldafny.generated.Sorting as Sorting
import smithy_dafny_standard_library.internaldafny.generated.SortedSets as SortedSets
import smithy_dafny_standard_library.internaldafny.generated.HexStrings as HexStrings
import smithy_dafny_standard_library.internaldafny.generated.GetOpt as GetOpt
import smithy_dafny_standard_library.internaldafny.generated.FloatCompare as FloatCompare
import smithy_dafny_standard_library.internaldafny.generated.ConcurrentCall as ConcurrentCall
import smithy_dafny_standard_library.internaldafny.generated.Base64 as Base64
import smithy_dafny_standard_library.internaldafny.generated.Base64Lemmas as Base64Lemmas
import smithy_dafny_standard_library.internaldafny.generated.Actions as Actions
import smithy_dafny_standard_library.internaldafny.generated.DafnyLibraries as DafnyLibraries
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views_Core as JSON_Utils_Views_Core
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views_Writers as JSON_Utils_Views_Writers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views as JSON_Utils_Views
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers_Core as JSON_Utils_Lexers_Core
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers_Strings as JSON_Utils_Lexers_Strings
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers as JSON_Utils_Lexers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Cursors as JSON_Utils_Cursors
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Parsers as JSON_Utils_Parsers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str_CharStrConversion as JSON_Utils_Str_CharStrConversion
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str_CharStrEscaping as JSON_Utils_Str_CharStrEscaping
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str as JSON_Utils_Str
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Seq as JSON_Utils_Seq
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Vectors as JSON_Utils_Vectors
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils as JSON_Utils
import smithy_dafny_standard_library.internaldafny.generated.JSON_Errors as JSON_Errors
import smithy_dafny_standard_library.internaldafny.generated.JSON_Values as JSON_Values
import smithy_dafny_standard_library.internaldafny.generated.JSON_Spec as JSON_Spec
import smithy_dafny_standard_library.internaldafny.generated.JSON_Grammar as JSON_Grammar
import smithy_dafny_standard_library.internaldafny.generated.JSON_Serializer_ByteStrConversion as JSON_Serializer_ByteStrConversion

# Module: JSON_Serializer

class default__:
    def  __init__(self):
        pass

    @staticmethod
    def Bool(b):
        return JSON_Utils_Views_Core.View__.OfBytes((JSON_Grammar.default__.TRUE if b else JSON_Grammar.default__.FALSE))

    @staticmethod
    def CheckLength(s, err):
        return Wrappers.default__.Need((len(s)) < (BoundedInts.default__.TWO__TO__THE__32), err)

    @staticmethod
    def String(str):
        d_0_valueOrError0_ = JSON_Spec.default__.EscapeToUTF8(str, 0)
        if (d_0_valueOrError0_).IsFailure():
            return (d_0_valueOrError0_).PropagateFailure()
        elif True:
            d_1_bs_ = (d_0_valueOrError0_).Extract()
            d_2_valueOrError1_ = default__.CheckLength(d_1_bs_, JSON_Errors.SerializationError_StringTooLong(str))
            if (d_2_valueOrError1_).IsFailure():
                return (d_2_valueOrError1_).PropagateFailure()
            elif True:
                return Wrappers.Result_Success(JSON_Grammar.jstring_JString(JSON_Grammar.default__.DOUBLEQUOTE, JSON_Utils_Views_Core.View__.OfBytes(d_1_bs_), JSON_Grammar.default__.DOUBLEQUOTE))

    @staticmethod
    def Sign(n):
        return JSON_Utils_Views_Core.View__.OfBytes((_dafny.Seq([ord('-')]) if (n) < (0) else _dafny.Seq([])))

    @staticmethod
    def Int_k(n):
        return JSON_Serializer_ByteStrConversion.default__.OfInt__any(n, default__.DIGITS, default__.MINUS)

    @staticmethod
    def Int(n):
        d_0_bs_ = default__.Int_k(n)
        d_1_valueOrError0_ = default__.CheckLength(d_0_bs_, JSON_Errors.SerializationError_IntTooLarge(n))
        if (d_1_valueOrError0_).IsFailure():
            return (d_1_valueOrError0_).PropagateFailure()
        elif True:
            return Wrappers.Result_Success(JSON_Utils_Views_Core.View__.OfBytes(d_0_bs_))

    @staticmethod
    def Number(dec):
        pat_let_tv0_ = dec
        pat_let_tv1_ = dec
        d_0_minus_ = default__.Sign((dec).n)
        d_1_valueOrError0_ = default__.Int(Math.default__.Abs((dec).n))
        if (d_1_valueOrError0_).IsFailure():
            return (d_1_valueOrError0_).PropagateFailure()
        elif True:
            d_2_num_ = (d_1_valueOrError0_).Extract()
            d_3_frac_ = JSON_Grammar.Maybe_Empty()
            def iife0_(_pat_let6_0):
                def iife1_(d_5_e_):
                    def iife2_(_pat_let7_0):
                        def iife3_(d_6_sign_):
                            def iife4_(_pat_let8_0):
                                def iife5_(d_7_valueOrError2_):
                                    def iife6_(_pat_let9_0):
                                        def iife7_(d_8_num_):
                                            return Wrappers.Result_Success(JSON_Grammar.Maybe_NonEmpty(JSON_Grammar.jexp_JExp(d_5_e_, d_6_sign_, d_8_num_)))
                                        return iife7_(_pat_let9_0)
                                    return ((d_7_valueOrError2_).PropagateFailure() if (d_7_valueOrError2_).IsFailure() else iife6_((d_7_valueOrError2_).Extract()))
                                return iife5_(_pat_let8_0)
                            return iife4_(default__.Int(Math.default__.Abs((pat_let_tv1_).e10)))
                        return iife3_(_pat_let7_0)
                    return iife2_(default__.Sign((pat_let_tv0_).e10))
                return iife1_(_pat_let6_0)
            d_4_valueOrError1_ = (Wrappers.Result_Success(JSON_Grammar.Maybe_Empty()) if ((dec).e10) == (0) else iife0_(JSON_Utils_Views_Core.View__.OfBytes(_dafny.Seq([ord('e')]))))
            if (d_4_valueOrError1_).IsFailure():
                return (d_4_valueOrError1_).PropagateFailure()
            elif True:
                d_9_exp_ = (d_4_valueOrError1_).Extract()
                return Wrappers.Result_Success(JSON_Grammar.jnumber_JNumber(d_0_minus_, d_2_num_, JSON_Grammar.Maybe_Empty(), d_9_exp_))

    @staticmethod
    def MkStructural(v):
        return JSON_Grammar.Structural_Structural(JSON_Grammar.default__.EMPTY, v, JSON_Grammar.default__.EMPTY)

    @staticmethod
    def KeyValue(kv):
        d_0_valueOrError0_ = default__.String((kv)[0])
        if (d_0_valueOrError0_).IsFailure():
            return (d_0_valueOrError0_).PropagateFailure()
        elif True:
            d_1_k_ = (d_0_valueOrError0_).Extract()
            d_2_valueOrError1_ = default__.Value((kv)[1])
            if (d_2_valueOrError1_).IsFailure():
                return (d_2_valueOrError1_).PropagateFailure()
            elif True:
                d_3_v_ = (d_2_valueOrError1_).Extract()
                return Wrappers.Result_Success(JSON_Grammar.jKeyValue_KeyValue(d_1_k_, default__.COLON, d_3_v_))

    @staticmethod
    def MkSuffixedSequence(ds, suffix, start):
        d_0___accumulator_ = _dafny.Seq([])
        while True:
            with _dafny.label():
                if (start) >= (len(ds)):
                    return (d_0___accumulator_) + (_dafny.Seq([]))
                elif (start) == ((len(ds)) - (1)):
                    return (d_0___accumulator_) + (_dafny.Seq([JSON_Grammar.Suffixed_Suffixed((ds)[start], JSON_Grammar.Maybe_Empty())]))
                elif True:
                    d_0___accumulator_ = (d_0___accumulator_) + (_dafny.Seq([JSON_Grammar.Suffixed_Suffixed((ds)[start], JSON_Grammar.Maybe_NonEmpty(suffix))]))
                    in0_ = ds
                    in1_ = suffix
                    in2_ = (start) + (1)
                    ds = in0_
                    suffix = in1_
                    start = in2_
                    raise _dafny.TailCall()
                break

    @staticmethod
    def Object(obj):
        def lambda0_(d_1_obj_):
            def lambda1_(d_2_v_):
                return default__.KeyValue(d_2_v_)

            return lambda1_

        d_0_valueOrError0_ = Seq.default__.MapWithResult(lambda0_(obj), obj)
        if (d_0_valueOrError0_).IsFailure():
            return (d_0_valueOrError0_).PropagateFailure()
        elif True:
            d_3_items_ = (d_0_valueOrError0_).Extract()
            return Wrappers.Result_Success(JSON_Grammar.Bracketed_Bracketed(default__.MkStructural(JSON_Grammar.default__.LBRACE), default__.MkSuffixedSequence(d_3_items_, default__.COMMA, 0), default__.MkStructural(JSON_Grammar.default__.RBRACE)))

    @staticmethod
    def Array(arr):
        def lambda0_(d_1_arr_):
            def lambda1_(d_2_v_):
                return default__.Value(d_2_v_)

            return lambda1_

        d_0_valueOrError0_ = Seq.default__.MapWithResult(lambda0_(arr), arr)
        if (d_0_valueOrError0_).IsFailure():
            return (d_0_valueOrError0_).PropagateFailure()
        elif True:
            d_3_items_ = (d_0_valueOrError0_).Extract()
            return Wrappers.Result_Success(JSON_Grammar.Bracketed_Bracketed(default__.MkStructural(JSON_Grammar.default__.LBRACKET), default__.MkSuffixedSequence(d_3_items_, default__.COMMA, 0), default__.MkStructural(JSON_Grammar.default__.RBRACKET)))

    @staticmethod
    def Value(js):
        source0_ = js
        if True:
            if source0_.is_Null:
                return Wrappers.Result_Success(JSON_Grammar.Value_Null(JSON_Utils_Views_Core.View__.OfBytes(JSON_Grammar.default__.NULL)))
        if True:
            if source0_.is_Bool:
                d_0_b_ = source0_.b
                return Wrappers.Result_Success(JSON_Grammar.Value_Bool(default__.Bool(d_0_b_)))
        if True:
            if source0_.is_String:
                d_1_str_ = source0_.str
                d_2_valueOrError0_ = default__.String(d_1_str_)
                if (d_2_valueOrError0_).IsFailure():
                    return (d_2_valueOrError0_).PropagateFailure()
                elif True:
                    d_3_s_ = (d_2_valueOrError0_).Extract()
                    return Wrappers.Result_Success(JSON_Grammar.Value_String(d_3_s_))
        if True:
            if source0_.is_Number:
                d_4_dec_ = source0_.num
                d_5_valueOrError1_ = default__.Number(d_4_dec_)
                if (d_5_valueOrError1_).IsFailure():
                    return (d_5_valueOrError1_).PropagateFailure()
                elif True:
                    d_6_n_ = (d_5_valueOrError1_).Extract()
                    return Wrappers.Result_Success(JSON_Grammar.Value_Number(d_6_n_))
        if True:
            if source0_.is_Object:
                d_7_obj_ = source0_.obj
                d_8_valueOrError2_ = default__.Object(d_7_obj_)
                if (d_8_valueOrError2_).IsFailure():
                    return (d_8_valueOrError2_).PropagateFailure()
                elif True:
                    d_9_o_ = (d_8_valueOrError2_).Extract()
                    return Wrappers.Result_Success(JSON_Grammar.Value_Object(d_9_o_))
        if True:
            d_10_arr_ = source0_.arr
            d_11_valueOrError3_ = default__.Array(d_10_arr_)
            if (d_11_valueOrError3_).IsFailure():
                return (d_11_valueOrError3_).PropagateFailure()
            elif True:
                d_12_a_ = (d_11_valueOrError3_).Extract()
                return Wrappers.Result_Success(JSON_Grammar.Value_Array(d_12_a_))

    @staticmethod
    def JSON(js):
        d_0_valueOrError0_ = default__.Value(js)
        if (d_0_valueOrError0_).IsFailure():
            return (d_0_valueOrError0_).PropagateFailure()
        elif True:
            d_1_val_ = (d_0_valueOrError0_).Extract()
            return Wrappers.Result_Success(default__.MkStructural(d_1_val_))

    @_dafny.classproperty
    def DIGITS(instance):
        return _dafny.Seq([ord('0'), ord('1'), ord('2'), ord('3'), ord('4'), ord('5'), ord('6'), ord('7'), ord('8'), ord('9')])
    @_dafny.classproperty
    def MINUS(instance):
        return ord('-')
    @_dafny.classproperty
    def COLON(instance):
        return default__.MkStructural(JSON_Grammar.default__.COLON)
    @_dafny.classproperty
    def COMMA(instance):
        return default__.MkStructural(JSON_Grammar.default__.COMMA)

class bytes32:
    def  __init__(self):
        pass

    @staticmethod
    def default():
        return _dafny.Seq({})
    def _Is(source__):
        d_0_bs_: _dafny.Seq = source__
        return (len(d_0_bs_)) < (BoundedInts.default__.TWO__TO__THE__32)

class string32:
    def  __init__(self):
        pass

    @staticmethod
    def default():
        return _dafny.Seq("")
    def _Is(source__):
        d_1_s_: _dafny.Seq = source__
        return (len(d_1_s_)) < (BoundedInts.default__.TWO__TO__THE__32)
